#include <Python.h>
#include <math.h>

typedef float (*FadeoutCalc)(float, float, float);

float fadeA (float d, float v1, float v2) {
	return powf(0.001, (v1+d)/v2);
}

float fadeB (float d, float v1, float v2) {
	return 1.0 - (v1+d)/(v2);
}

float fadeC (float d, float v1, float v2) {
	d = (v1+d)/v2;
	return (powf(0.001, d)+(1.0-d))/2.0;
}

float fadeD (float d, float v1, float v2) {
	return (cosf(M_PI * (v1+d)/(v2))+1.0)/2.0;
}

static PyObject *
do_fadeout(PyObject *self, PyObject *args)
{
	char *wav;
	int wavLen, hLen, cur, meth;
	short int *samp;
	float d, v1, v2;
	FadeoutCalc fadefunc;
	
	if (!PyArg_ParseTuple(args, "s#iff", &wav, &wavLen, &meth, &v1, &v2)) {
		return NULL;
	}

	switch (meth){
		case 0:
			fadefunc = fadeA;
			break;
		case 1:
			fadefunc = fadeB;
			break;
		case 2:
			fadefunc = fadeC;
			break;
		default:
			fadefunc = fadeD;
			break;
	}

	hLen = wavLen / 2;
	samp = (short *)wav;
	
	for (cur=0; cur<hLen; cur+=2) {
		d = fadefunc(cur>>1, v1, v2);
		samp[cur] *= d;
		samp[cur+1] *= d;
	}
	return Py_BuildValue("s#", wav, wavLen);
}

static PyMethodDef FadeoutMethods[] = {
	{"fadeout", do_fadeout, METH_VARARGS, "Fade out a byte array"},
	{NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
initfadeout (void)
{
	Py_InitModule("fadeout", FadeoutMethods);
}
